#ifndef _GvsPreprocessor_H__
#define _GvsPreprocessor_H__

#include <fstream>
#include <stack>
using namespace std;

#include "Preprocessor.h"

namespace GtpVisibilityPreprocessor {

class Exporter;
class VssRay;


/** Sampling based visibility preprocessing. The implementation is 
	based on heuristical sampling of view space.
*/
class GvsPreprocessor : public Preprocessor {

public:

	GvsPreprocessor();
	~GvsPreprocessor() {}

	virtual bool ComputeVisibility();
	

protected:

	struct GvsRayInfo
	{
		VssRay *mRay;
		bool mFoundDiscontinuity;
	}

	
	typedef stack<GvsRayInfo> RayQueue;

	/** Runs the adaptive sampling. The method starts with a number of random rays given
		by the queue and continues as long it finds new visible geometry (i.e., the queue is
		not empty).

		@returns the number of samples cast.
	*/
	int RunSampling();

	/** One pass of the sampling preprocessor. Continues as long as at least passSample
		rays have been cast.
		@returns the number of samples cast.
	*/
	int Pass();

	/** Generates the rays starting the adaptive visibility sampling process.
	*/
	int CastAvsSamples(
		const int samplesPerPass, 
		const int sampleType, 
		RayQueue &passSamples);

	/** Uses the information gained from the ray for doing adaptive border sampling.
		This function tries to find the border of the triangle found visible by the
		current ray. New rays are generated which sample this border.
		
		We use the following strategies:

		a) if new triangle was found: adaptive border sampling
		b) if triangle was found reverse sampling
	*/
	int HandleRay(const GvsRayInfo &ray);

	/** 
	*/	
	int AdaptiveBorderSampling(const VssRay &ray);
	
	int ReverseSampling(const VssRay &ray);

	/** Cast samples according to a specific sampling strategy.
	*/
	int CastPassSamples(
		const int samplesPerPass, 
		const int sampleType, 
		VssRayContainer &passSamples) const;

	void CreateNewSamples(
		VertexContainer &samples, 
		const VssRay &ray, 
		const Triangle3 &hitTriangle, 
		const int index,
		const float eps);

	const bool DiscontinuityFound(const VssRay &ray) const;


	//////////////////////

	ofstream mStats;

	int mSamplesPerPass;
	RayQueue mRayQueue; 
	int mSamplingType;
	int mTotalSamples;
	int mInitialSamples;
	AxisAlignedBox3 mViewSpaceBox;
	float mEps;
};

};

#endif
